@@ -62,7 +62,8 @@ json_object_new (void)
   JsonObject *object;
 
   object = g_slice_new0 (JsonObject);
-  
+
+  object->age = 0;
   object->ref_count = 1;
   object->members = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            g_free,
@@ -176,7 +177,10 @@ object_set_member_internal (JsonObject  *object,
   gchar *name = g_strdup (member_name);
 
   if (g_hash_table_lookup (object->members, name) == NULL)
-    g_queue_push_tail (&object->members_ordered, name);
+    {
+      g_queue_push_tail (&object->members_ordered, name);
+      object->age += 1;
+    }
   else
     {
       GList *l;
@@ -567,7 +571,7 @@ object_get_member_internal (JsonObject  *object,
  * a #JsonObject.
  *
  * Return value: (transfer none) (nullable): a pointer to the node for the
- *   requested object member, or %NULL
+ *   requested object member, or %NULL if it does not exist.
  */
 JsonNode *
 json_object_get_member (JsonObject  *object,
@@ -579,35 +583,82 @@ json_object_get_member (JsonObject  *object,
   return object_get_member_internal (object, member_name);
 }
 
+#define JSON_OBJECT_GET(ret_type,type_name) \
+ret_type \
+json_object_get_ ##type_name## _member (JsonObject *object, \
+                                        const char *member_name) \
+{ \
+  g_return_val_if_fail (object != NULL, (ret_type) 0); \
+  g_return_val_if_fail (member_name != NULL, (ret_type) 0); \
+\
+  JsonNode *node = object_get_member_internal (object, member_name); \
+  g_return_val_if_fail (node != NULL, (ret_type) 0); \
+\
+  if (JSON_NODE_HOLDS_NULL (node)) \
+    return (ret_type) 0; \
+\
+  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, (ret_type) 0); \
+\
+  return json_node_get_ ##type_name (node); \
+}
+
+#define JSON_OBJECT_GET_DEFAULT(ret_type,type_name) \
+ret_type \
+json_object_get_ ##type_name## _member_with_default (JsonObject *object, \
+                                                     const char *member_name, \
+                                                     ret_type    default_value) \
+{ \
+  g_return_val_if_fail (object != NULL, default_value); \
+  g_return_val_if_fail (member_name != NULL, default_value); \
+\
+  JsonNode *node = object_get_member_internal (object, member_name); \
+  if (node == NULL) \
+    return default_value; \
+\
+  if (JSON_NODE_HOLDS_NULL (node)) \
+    return default_value; \
+\
+  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, default_value); \
+\
+  return json_node_get_ ##type_name (node); \
+}
+
 /**
  * json_object_get_int_member:
  * @object: a #JsonObject
- * @member_name: the name of the member
+ * @member_name: the name of the @object member
  *
  * Convenience function that retrieves the integer value
- * stored in @member_name of @object
+ * stored in @member_name of @object. It is an error to specify a
+ * @member_name which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_int_member_with_default(),
+ * json_object_get_member() and json_object_has_member()
  *
  * Return value: the integer value of the object's member
  *
  * Since: 0.8
  */
-gint64
-json_object_get_int_member (JsonObject  *object,
-                            const gchar *member_name)
-{
-  JsonNode *node;
+JSON_OBJECT_GET (gint64, int)
 
-  g_return_val_if_fail (object != NULL, 0);
-  g_return_val_if_fail (member_name != NULL, 0);
-
-  node = object_get_member_internal (object, member_name);
-  g_return_val_if_fail (node != NULL, 0);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, 0);
-
-  return json_node_get_int (node);
-}
+/**
+ * json_object_get_int_member_with_default:
+ * @object: a #JsonObject
+ * @member_name: the name of the @object member
+ * @default_value: the value to return if @member_name is not valid
+ *
+ * Convenience function that retrieves the integer value
+ * stored in @member_name of @object.
+ *
+ * If @member_name does not exist, does not contain a scalar value,
+ * or contains `null`, then @default_value is returned instead.
+ *
+ * Returns: the integer value of the object's member, or the
+ *   given default
+ *
+ * Since: 1.6
+ */
+JSON_OBJECT_GET_DEFAULT (gint64, int)
 
 /**
  * json_object_get_double_member:
@@ -615,29 +666,36 @@ json_object_get_int_member (JsonObject  *object,
  * @member_name: the name of the member
  *
  * Convenience function that retrieves the floating point value
- * stored in @member_name of @object
+ * stored in @member_name of @object. It is an error to specify a
+ * @member_name which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_double_member_with_default(),
+ * json_object_get_member() and json_object_has_member()
  *
  * Return value: the floating point value of the object's member
  *
  * Since: 0.8
  */
-gdouble
-json_object_get_double_member (JsonObject  *object,
-                               const gchar *member_name)
-{
-  JsonNode *node;
+JSON_OBJECT_GET (gdouble, double)
 
-  g_return_val_if_fail (object != NULL, 0.0);
-  g_return_val_if_fail (member_name != NULL, 0.0);
-
-  node = object_get_member_internal (object, member_name);
-  g_return_val_if_fail (node != NULL, 0.0);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, 0.0);
-
-  return json_node_get_double (node);
-}
+/**
+ * json_object_get_double_member_with_default:
+ * @object: a #JsonObject
+ * @member_name: the name of the @object member
+ * @default_value: the value to return if @member_name is not valid
+ *
+ * Convenience function that retrieves the floating point value
+ * stored in @member_name of @object.
+ *
+ * If @member_name does not exist, does not contain a scalar value,
+ * or contains `null`, then @default_value is returned instead.
+ *
+ * Returns: the floating point value of the object's member, or the
+ *   given default
+ *
+ * Since: 1.6
+ */
+JSON_OBJECT_GET_DEFAULT (double, double)
 
 /**
  * json_object_get_boolean_member:
@@ -645,29 +703,73 @@ json_object_get_double_member (JsonObject  *object,
  * @member_name: the name of the member
  *
  * Convenience function that retrieves the boolean value
- * stored in @member_name of @object
+ * stored in @member_name of @object. It is an error to specify a
+ * @member_name which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_boolean_member_with_default(),
+ * json_object_get_member() and json_object_has_member()
  *
  * Return value: the boolean value of the object's member
  *
  * Since: 0.8
  */
-gboolean
-json_object_get_boolean_member (JsonObject  *object,
-                                const gchar *member_name)
-{
-  JsonNode *node;
+JSON_OBJECT_GET (gboolean, boolean)
 
-  g_return_val_if_fail (object != NULL, FALSE);
-  g_return_val_if_fail (member_name != NULL, FALSE);
+/**
+ * json_object_get_boolean_member_with_default:
+ * @object: a #JsonObject
+ * @member_name: the name of the @object member
+ * @default_value: the value to return if @member_name is not valid
+ *
+ * Convenience function that retrieves the boolean value
+ * stored in @member_name of @object.
+ *
+ * If @member_name does not exist, does not contain a scalar value,
+ * or contains `null`, then @default_value is returned instead.
+ *
+ * Returns: the boolean value of the object's member, or the
+ *   given default
+ *
+ * Since: 1.6
+ */
+JSON_OBJECT_GET_DEFAULT (gboolean, boolean)
 
-  node = object_get_member_internal (object, member_name);
-  g_return_val_if_fail (node != NULL, FALSE);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, FALSE);
+/**
+ * json_object_get_string_member:
+ * @object: a #JsonObject
+ * @member_name: the name of the member
+ *
+ * Convenience function that retrieves the string value
+ * stored in @member_name of @object. It is an error to specify a
+ * @member_name that does not exist.
+ *
+ * See also: json_object_get_string_member_with_default(),
+ * json_object_get_member() and json_object_has_member()
+ *
+ * Return value: the string value of the object's member
+ *
+ * Since: 0.8
+ */
+JSON_OBJECT_GET (const gchar *, string)
 
-  return json_node_get_boolean (node);
-}
+/**
+ * json_object_get_string_member_with_default:
+ * @object: a #JsonObject
+ * @member_name: the name of the @object member
+ * @default_value: the value to return if @member_name is not valid
+ *
+ * Convenience function that retrieves the string value
+ * stored in @member_name of @object.
+ *
+ * If @member_name does not exist, does not contain a scalar value,
+ * or contains `null`, then @default_value is returned instead.
+ *
+ * Returns: the string value of the object's member, or the
+ *   given default
+ *
+ * Since: 1.6
+ */
+JSON_OBJECT_GET_DEFAULT (const char *, string)
 
 /**
  * json_object_get_null_member:
@@ -675,9 +777,10 @@ json_object_get_boolean_member (JsonObject  *object,
  * @member_name: the name of the member
  *
  * Convenience function that checks whether the value
- * stored in @member_name of @object is null
+ * stored in @member_name of @object is null. It is an error to
+ * specify a @member_name which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_member() and json_object_has_member()
  *
  * Return value: %TRUE if the value is null
  *
@@ -707,48 +810,16 @@ json_object_get_null_member (JsonObject  *object,
   return FALSE;
 }
 
-/**
- * json_object_get_string_member:
- * @object: a #JsonObject
- * @member_name: the name of the member
- *
- * Convenience function that retrieves the string value
- * stored in @member_name of @object
- *
- * See also: json_object_get_member()
- *
- * Return value: the string value of the object's member
- *
- * Since: 0.8
- */
-const gchar *
-json_object_get_string_member (JsonObject  *object,
-                               const gchar *member_name)
-{
-  JsonNode *node;
-
-  g_return_val_if_fail (object != NULL, NULL);
-  g_return_val_if_fail (member_name != NULL, NULL);
-
-  node = object_get_member_internal (object, member_name);
-  g_return_val_if_fail (node != NULL, NULL);
-  g_return_val_if_fail (JSON_NODE_HOLDS_VALUE (node) || JSON_NODE_HOLDS_NULL (node), NULL);
-
-  if (JSON_NODE_HOLDS_NULL (node))
-    return NULL;
-
-  return json_node_get_string (node);
-}
-
 /**
  * json_object_get_array_member:
  * @object: a #JsonObject
  * @member_name: the name of the member
  *
  * Convenience function that retrieves the array
- * stored in @member_name of @object
+ * stored in @member_name of @object. It is an error to specify a
+ * @member_name which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_member() and json_object_has_member()
  *
  * Return value: (transfer none): the array inside the object's member
  *
@@ -782,7 +853,7 @@ json_object_get_array_member (JsonObject  *object,
  * stored in @member_name of @object. It is an error to specify a @member_name
  * which does not exist.
  *
- * See also: json_object_get_member()
+ * See also: json_object_get_member() and json_object_has_member()
  *
  * Return value: (transfer none) (nullable): the object inside the object’s
  *    member, or %NULL if the value for the member is `null`
@@ -886,6 +957,9 @@ json_object_remove_member (JsonObject  *object,
  * from within the iterator @func, but it is not safe to add or
  * remove members from the @object.
  *
+ * The order in which the @object members are iterated is the
+ * insertion order.
+ *
  * Since: 0.8
  */
 void
@@ -894,16 +968,21 @@ json_object_foreach_member (JsonObject        *object,
                             gpointer           data)
 {
   GList *l;
+  int age;
 
   g_return_if_fail (object != NULL);
   g_return_if_fail (func != NULL);
 
+  age = object->age;
+
   for (l = object->members_ordered.head; l != NULL; l = l->next)
     {
       const gchar *member_name = l->data;
       JsonNode *member_node = g_hash_table_lookup (object->members, member_name);
 
       func (object, member_name, member_node, data);
+
+      g_assert (object->age == age);
     }
 }
 
@@ -1053,6 +1132,12 @@ json_object_iter_init (JsonObjectIter  *iter,
  * The order in which members are returned by the iterator is undefined. The
  * iterator is invalidated if its #JsonObject is modified during iteration.
  *
+ * You must use this function with a #JsonObjectIter initialized with
+ * json_object_iter_init(); using this function with an iterator initialized
+ * with json_object_iter_init_ordered() yields undefined behavior.
+ *
+ * See also: json_object_iter_next_ordered()
+ *
  * Returns: %TRUE if @member_name and @member_node are valid; %FALSE if the end
  *    of the object has been reached
  *
@@ -1073,3 +1158,102 @@ json_object_iter_next (JsonObjectIter  *iter,
                                  (gpointer *) member_name,
                                  (gpointer *) member_node);
 }
+
+/**
+ * json_object_iter_init_ordered:
+ * @iter: an uninitialised #JsonObjectIter
+ * @object: the #JsonObject to iterate over
+ *
+ * Initialise the @iter and associate it with @object.
+ *
+ * |[<!-- language="C" -->
+ * JsonObjectIter iter;
+ * const gchar *member_name;
+ * JsonNode *member_node;
+ *
+ * json_object_iter_init_ordered (&iter, some_object);
+ * while (json_object_iter_next_ordered (&iter, &member_name, &member_node))
+ *   {
+ *     // Do something with @member_name and @member_node.
+ *   }
+ * ]|
+ *
+ * See also: json_object_iter_init()
+ *
+ * Since: 1.6
+ */
+void
+json_object_iter_init_ordered (JsonObjectIter  *iter,
+                               JsonObject      *object)
+{
+  JsonObjectOrderedIterReal *iter_real = (JsonObjectOrderedIterReal *) iter;
+
+  g_return_if_fail (iter != NULL);
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->ref_count > 0);
+
+  iter_real->object = object;
+  iter_real->cur_member = NULL;
+  iter_real->next_member = NULL;
+  iter_real->age = iter_real->object->age;
+}
+
+/**
+ * json_object_iter_next_ordered:
+ * @iter: a #JsonObjectIter
+ * @member_name: (out callee-allocates) (transfer none) (optional): return
+ *    location for the member name, or %NULL to ignore
+ * @member_node: (out callee-allocates) (transfer none) (optional): return
+ *    location for the member value, or %NULL to ignore
+ *
+ * Advance @iter and retrieve the next member in the object. If the end of the
+ * object is reached, %FALSE is returned and @member_name and @member_node are
+ * set to invalid values. After that point, the @iter is invalid.
+ *
+ * The order in which members are returned by the iterator is the same order in
+ * which the members were added to the #JsonObject. The iterator is invalidated
+ * if its #JsonObject is modified during iteration.
+ *
+ * You must use this function with a #JsonObjectIter initialized with
+ * json_object_iter_init_ordered(); using this function with an iterator initialized
+ * with json_object_iter_init() yields undefined behavior.
+ *
+ * See also: json_object_iter_next()
+ *
+ * Returns: %TRUE if @member_name and @member_node are valid; %FALSE if the end
+ *    of the object has been reached
+ *
+ * Since: 1.6
+ */
+gboolean
+json_object_iter_next_ordered (JsonObjectIter  *iter,
+                               const gchar    **member_name,
+                               JsonNode       **member_node)
+{
+  JsonObjectOrderedIterReal *iter_real = (JsonObjectOrderedIterReal *) iter;
+  const char *name = NULL;
+
+  g_return_val_if_fail (iter != NULL, FALSE);
+  g_return_val_if_fail (iter_real->object != NULL, FALSE);
+  g_return_val_if_fail (iter_real->object->ref_count > 0, FALSE);
+  g_return_val_if_fail (iter_real->age == iter_real->object->age, FALSE);
+
+  if (iter_real->cur_member == NULL)
+    iter_real->cur_member = iter_real->object->members_ordered.head;
+  else
+    iter_real->cur_member = iter_real->cur_member->next;
+
+  name = iter_real->cur_member != NULL ? iter_real->cur_member->data : NULL;
+
+  if (member_name != NULL)
+    *member_name = name;
+  if (member_node != NULL)
+    {
+      if (name != NULL)
+        *member_node = g_hash_table_lookup (iter_real->object->members, name);
+      else
+        *member_name = NULL;
+    }
+
+  return iter_real->cur_member != NULL;
+}
